package org.eclipse.gmt.gems.css.parser;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.imageio.stream.FileImageInputStream;

import org.apache.commons.io.IOUtils;
import org.eclipse.gmt.gems.css.parser.Attributes;
import org.eclipse.gmt.gems.css.parser.CSSParser;

/******************************************************************************
 * Copyright (c) 2005, 2006 Jules White.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Jules White - initial API and implementation 
 ****************************************************************************/
public class AttributesManager {

	private static AttributesManager instance_;

	public static synchronized AttributesManager getInstance() {
		if (instance_ == null)
			instance_ = new AttributesManager();

		return instance_;
	}

	private Map<String, List<Attributes>> attributeCache_ = new HashMap<String, List<Attributes>>(
			51);

	private Map<String, Object> attributeObjCache_ = new HashMap<String, Object>(
			51);

	private Map<String, AttributesConverter> converters_ = new HashMap<String, AttributesConverter>();

	private AttributesManager() {
	}

	private String key(String id, String attrs) {
		return id + "::" + attrs;
	}

	public void addConverter(String type, AttributesConverter c) {
		converters_.put(type, c);
	}

	public void removeConverter(String type) {
		converters_.remove(type);
	}

	public <AType> AType getAttributesObject(String id, File base,
			String attrtypes, Class<AType> c) {
		String key = key(id, attrtypes);
		Object obj = attributeObjCache_.get(key);
		if (obj == null) {
			AttributesConverter converter = converters_.get(attrtypes);
			if (converter != null) {
				obj = converter.convert(getAttributes(id, base, attrtypes));
				attributeObjCache_.put(key, obj);
			}
		}
		return c.cast(obj);
	}

	public List<Attributes> getAttributes(String id, File base, String attrtypes) {
		String key = key(id, attrtypes);
		List<Attributes> attrs = attributeCache_.get(key);
		if (attrs == null) {
			attrs = loadAttributes(base, attrtypes);
			attributeCache_.put(key, attrs);
		}
		return attrs;
	}
	
	public List<Attributes> getAttributes(String id, String attrtypes) {
		return attributeCache_.get(key(id,attrtypes));
	}
	
	public <AType> AType getAttributesObject(String id, String attrtypes, Class<AType> c) {
		Object obj = attributeObjCache_.get(key(id,attrtypes));
		if(obj != null)
			return c.cast(obj);
		return null;
	}
	
	public Object getAttributesObject(String id, String attrtypes) {
		Object obj = attributeObjCache_.get(key(id,attrtypes));
		if(obj != null)
			return obj;
		return null;
	}
	
	public void flushCache(){
		attributeCache_.clear();
		attributeObjCache_.clear();
	}

	public void flushCache(String id, String attr) {
		String key = key(id, attr);
		attributeCache_.remove(key);
		attributeObjCache_.remove(key);
	}

	public void flushCache(String id) {
		for(Object o : attributeCache_.keySet()){
			String key = ""+o;
			if(key.startsWith(id)){
				attributeCache_.remove(key);
				attributeObjCache_.remove(key);
			}
		}		
	}
	
	public List<Attributes> loadAttributes(File mfile, String attrtypes) {
		ArrayList<Attributes> attrs = new ArrayList<Attributes>();
		try {
			File afile = new File(mfile.getParentFile(), mfile.getName() + "."
					+ attrtypes);
			if (afile.exists()) {
				FileInputStream fin = new FileInputStream(afile);
				String css = IOUtils.toString(fin);
				fin.close();
				return CSSParser.parseAttributes(css);
			}
		} catch (Exception e) {
			
		}
		return attrs;
	}

	public <AType> AType getAttributesObject(String id, InputStream base,
			String attrtypes, Class<AType> c) {
		String key = key(id, attrtypes);
		Object obj = attributeObjCache_.get(key);
		if (obj == null) {
			AttributesConverter converter = converters_.get(attrtypes);
			if (converter != null) {
				obj = converter.convert(getAttributes(id, base, attrtypes));
				attributeObjCache_.put(key, obj);
			}
		}
		return c.cast(obj);
	}

	public List<Attributes> getAttributes(String id, InputStream base,
			String attrtypes) {
		return getAttributes(id, base, attrtypes,false,true);
	}
	
	public List<Attributes> getAttributes(String id, InputStream base,
			String attrtypes, boolean preserveescapes, boolean overwritedups) {
		String key = key(id, attrtypes);
		List<Attributes> attrs = attributeCache_.get(key);
		if (attrs == null) {
			attrs = loadAttributes(id, base, attrtypes, preserveescapes, overwritedups);
			attributeCache_.put(key, attrs);
		}
		return attrs;
	}

	public List<Attributes> loadAttributes(String id, InputStream fin,
			String attrtypes) {
		return loadAttributes(id, fin, attrtypes,false,true);
	}
	
	public List<Attributes> loadAttributes(String id, InputStream fin,
			String attrtypes, boolean preserveescapes, boolean overwritedups) {
		ArrayList<Attributes> attrs = new ArrayList<Attributes>();
		if (fin != null) {
			try {
				String css = IOUtils.toString(fin);
				fin.close();
				return CSSParser.parseAttributes(css,preserveescapes,overwritedups);
			} catch (Exception e) {
				
			}
		}
		return attrs;
	}
}
