package org.eclipse.gmt.gems.css.parser;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/******************************************************************************
 * Copyright (c) 2005, 2006 Jules White.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Jules White - initial API and implementation 
 ****************************************************************************/
public class CSSParser {
	
	public static List<Attributes> parseAttributeSets(String styles){
		return parseAttributeSets(styles,false,true);
	}
	
	
	public static List<Attributes> parseAttributeSets(String styles, boolean keepescapes, boolean overwritedups){
		styles = styles.trim();
		styles = styles.replaceAll("/\\*[^/]*?\\*/", "");
		ArrayList<Attributes> sets = new ArrayList<Attributes>();
		StringTokenizer tk = new StringTokenizer(styles,"{}:;",true);
		while(tk.hasMoreTokens()){
			sets.add(parseAttributes(tk,keepescapes,overwritedups));
		}
		return sets;
	}
	
	public static Attributes parseAttributes(StringTokenizer style){
		return parseAttributes(style, false, true);
	}
	
	public static Attributes parseAttributes(StringTokenizer style, boolean preserveescpaes, boolean overwritedups){
		String selector = style.nextToken().trim();
	
		//remove the first {
		style.nextToken();
		
		boolean inval = false;
		String token = null;
		String key = null;
		String val = "";
		Attributes props = new Attributes(selector);
		while(style.hasMoreTokens()){
			
			token = style.nextToken();
			
			token = token.trim();
			if(token.length() < 1)
				continue;
			
			if(token.equals("}")){
				break;
			}
			if(token.equals(":"))
				inval = true;
			if(token.equals(";")){
				//props.put(key,val.trim());
				setProperty(key.trim(), val.trim(), props, overwritedups);
				key = null;
				inval = false;
				val = "";
				continue;
			}
			if(token.equals("{") && inval){
				token = parseEscapedValue(style);
				if(preserveescpaes){
					token = "{"+token+"}";
				}
			}
			if(!token.equals(";")
					&& !token.equals(":")){
				if(key == null){
					key = token.trim();
				}
				else{
					val += token;
					
				}
			}
			
		}
		
		return props;
	}
	
	public static void setProperty(String prop, Object val, Attributes props, boolean overwritedups){
		if(overwritedups){
			props.put(prop.trim(), val);
		}
		else{
			Object eval = props.get(prop);
			if(eval != null){
				if(eval instanceof List){
					((List)eval).add(val);
				}
				else{
					ArrayList l = new ArrayList();
					l.add(eval);
					l.add(val);
					props.put(prop, l);
				}
			}
			else{
				ArrayList l = new ArrayList();
				l.add(val);
				props.put(prop, l);
			}
		}
	}
	
	public static String parseEscapedValue(StringTokenizer tk){
		int lhs = 1;
		
		String val = "";
		while(lhs > 0 && tk.hasMoreTokens()){
			String token = tk.nextToken();
			if(token.equals("{"))
				lhs++;
			else if(token.equals("}"))
				lhs--;
			else
				val += token;
		}
		return val;
	}
	
	public static List<Attributes> parseAttributes(String reqcss, boolean preserveescapes, boolean overwritedups){
		
		List<Attributes> attrs = parseAttributeSets(reqcss,preserveescapes,overwritedups);
		return attrs;
	}

	public static List<Attributes> parseAttributes(String reqcss){
		return parseAttributes(reqcss,false,true);
	}
	

}

